
# q-expansion of newform 2100.2.a.b, downloaded from the LMFDB on 08 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2100
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [0], [-1], [-3], [4], [6], [-4], [-3], [3], [-10], [7], [0], [1], [12], [-6], [12], [-4], [7], [9], [-2], [17], [6], [0], [-2], [6], [4], [-12], [-7], [15], [7], [-6], [18], [14], [9], [-13], [4], [4], [24], [-24], [12], [14], [12], [25], [3], [-16], [-4], [-2], [0], [-10], [9], [24], [-16], [6], [6], [3], [-30], [-16], [-2], [15], [-26], [0], [-26], [-6], [-20], [3], [17], [-26], [-9], [-28], [-18], [33], [4], [37], [-31], [-12], [-33], [22], [27], [8], [30], [-1], [0], [16], [-4], [-12], [-39], [-35], [0], [4], [-18], [12], [-11], [33], [20], [6], [-36], [-18], [-8], [-25], [19], [27], [-42], [15], [41], [-8], [-18], [12], [-9], [44], [40], [-29], [-27], [-40], [5], [-45], [4], [42], [6], [12], [32], [46], [6], [15], [-28], [-30], [38], [0], [-14], [4], [-25], [0], [-40], [-17], [12], [32], [42], [-2], [30], [45], [-22], [42], [-5], [3], [-34], [54], [4], [12], [14], [51], [-14], [-54], [-41], [-12], [-8], [27], [-19], [-42], [52], [-42], [-60], [-57], [40], [-24], [45], [42], [-19], [28], [-1], [-30], [-33], [20], [-57], [46], [14], [-6], [-7], [-42], [-20], [26], [61], [-60], [7], [-30], [33], [-6], [1], [34], [53], [6], [-14], [33], [47], [18], [-45], [12], [41], [-17], [60], [-6], [30], [26], [22], [-28], [0], [60], [-22], [12], [9], [8], [67], [-30], [-23], [-36], [6], [-10], [-23], [-60], [-3], [9], [17], [-34], [-3], [-29], [36], [-10], [-18], [63], [-2], [-51], [-5], [-34], [32], [-51], [-44], [-30], [68], [-9], [-63], [-60], [-15], [44], [34], [35], [-24], [-54], [22], [42], [8], [48], [-50], [24], [48], [50], [24], [-60], [-19], [-26], [60], [-20], [16], [-9], [50], [-38], [-30], [-64], [57], [18], [49], [9], [68], [52], [-8], [-7], [-26], [82], [-57], [-34], [-49], [42], [-66], [-40], [12], [50], [-74], [57], [-47], [-33], [44], [-66], [-27], [-66], [-51], [-42], [-59], [30], [-52], [-36], [-36], [-26], [-50], [-78], [-79], [-9], [-67], [70], [45], [-16], [-9], [-2], [-6], [-30], [15], [-44], [-27], [-34], [6], [60], [-20], [-21], [26], [22], [-36], [-41], [63], [38], [-55], [-86], [-3], [21], [-22], [45], [2], [-18], [8], [72], [-55], [0], [-4], [34], [70], [-27], [72], [53], [-87], [-15], [-46], [73], [-48], [42], [14], [-47], [78], [4], [11], [-48], [6], [-42], [30], [-72], [-83], [-90], [36], [-39], [52], [10], [54], [61], [-61], [57], [-34], [-15], [-42], [50], [49], [60], [33], [-38], [84], [-86], [78], [-33], [43], [-21], [-22], [24], [95], [22], [-47], [72], [-43], [6], [57], [-2], [9], [-29], [32], [0], [68], [-33], [32], [39], [-17], [-84], [84], [-52], [61], [-78], [88], [54], [88], [57], [-96], [-31], [67], [60], [24], [-38], [60], [-42], [66], [70], [72], [-24], [-74], [-54], [27], [-66], [-46], [-6]]
